{-# LANGUAGE FlexibleContexts, TemplateHaskell, TypeFamilies, ScopedTypeVariables, ViewPatterns #-}
module AlignId where
import Generics.BiGUL
import Generics.BiGUL.Interpreter
import Generics.BiGUL.TH
import Generics.BiGUL.Lib
import Generics.BiGUL.Lib.List
import Data.List
import Data.Maybe
import GHC.Generics
import Debug.Trace
import Structure
import PRBAC
import System.IO  
import Control.Monad
import Text.Regex

keyid :: [(Key, Value)] -> String
keyid obj | length obj == 0 = ""
          | fst (head obj) == "pk" = getvaluestring (snd (head obj))
          | otherwise = keyid (tail obj)

keyids :: [Object] -> [String]
keyids objs | length objs == 0 = []
            | otherwise = keyid (head objs) : keyids (tail objs)

judgeobject :: [(Key, Value)] -> [(Key, Value)] -> Bool
judgeobject o1 o2 | length o1 == 0 && length o2 == 0 = True
                  | key1 == key2 && value1==value2 = judgeobject (tail o1) (tail o2)
                  | otherwise = False
         where key1 = fst (head o1)
               key2 = fst (head o2)
               value1 = getvaluestring (snd (head o1))
               value2 = getvaluestring (snd (head o2))

judgecondition :: Policy -> [(Key, Value)] -> Bool
judgecondition pol obj | length obj == 0 = True
                       | variable == "" = True
                       | fst (head obj) == variable = judgeequal variablevalue variableoperator (snd (head obj))
                       | otherwise = judgecondition pol (tail obj)
         where con = paracondition pol
               variable = fst (fst con)
               variablevalue = snd (fst con)
               variableoperator = snd con

judgeequal:: String->String->Value->Bool
judgeequal variablevalue variableoperator value
                 |  variableoperator == "NEqual" && variablevalue /= str = True
                 |  variableoperator == "NEqual" && variablevalue == str = False
                 |  variableoperator == "Equal" && variablevalue == str = True
                 |  variableoperator == "Equal" && variablevalue /= str = False
                    where str = getvaluestring value


extract :: String->[Object]->(Object,[Object])
extract k ss | keyid (head ss) == k = (head ss, tail ss)
             | otherwise = let (y,ys) = extract k (tail ss)
                               in (y,(head ss):ys)

alignread :: Policy -> BiGUL [Object] [Object]
alignread pol = Case
   [
      $(normalSV [p| [] |] [p| [] |] [p| [] |])
         ==> $(update [p| [] |] [p| [] |] [d| |] )
     ,$(normal [| \(s1:ss) (v1:vs) -> not (judgecondition pol v1) |] [| const False |] )
         ==> (Fail "the view is not legal (condition not satisfied)")
     ,$(normal [| \(s1:ss) (v1:vs) -> keyid s1 == keyid v1 && judgeobject s1 v1|] [|  \(s1:ss) -> (judgecondition pol s1) |] )
         ==> $(update [p| (x:xs) |] [p| x:xs |]
                      [d| x = Replace ; xs = alignread pol |] )
     ,$(normal [| \(s1:ss) (v1:vs) -> keyid s1 == keyid v1 && not (judgeobject s1 v1) |] [| const False |] )
         ==> (Fail "the view cannot be rewriten (different from source)")
     ,$(normal [| \ss (v1:vs) -> not ((keyid v1) `elem` (keyids ss)) |] [| const False |] )
         ==> (Fail "the view cannot be rewriten (with added information)")
     ,$(normal [| \(s1:ss) vs -> not (judgecondition pol s1) |] [| \(s1:ss) -> not (judgecondition pol s1) |])
         ==> $(update [p| _:ss  |] [p| ss |] [d| ss = alignread pol |] )
     ,$(adaptive[| \ss (v1:vs)-> (keyid v1) `elem` (keyids ss) |])
         ==> \ss (v:_) -> uncurry (:) (extract (keyid v) ss)
   ]

alignwrite :: Policy -> BiGUL [Object] [Object]
alignwrite pol = Case
   [
      $(normalSV [p| [] |] [p| [] |] [p| [] |])
         ==> $(update [p| [] |] [p| [] |] [d| |] )
     ,$(normal [| \(s1:ss) (v1:vs) -> not (judgecondition pol v1) |] [| const False |] )
         ==> (Fail "the view is not legal (condition not satisfied)")
     ,$(normal [| \(s1:ss) vs -> not (judgecondition pol s1) |] [| \(s1:ss) -> not (judgecondition pol s1) |])
         ==> $(update [p| _:ss  |] [p| ss |] [d| ss = alignwrite pol |] )
     ,$(normal [| \(s1:ss) (v1:vs) -> keyid s1 == keyid v1 |] [p| _:_ |] )
         ==> $(update [p| (x:xs) |] [p| x:xs |]
                      [d| x = Replace ; xs = alignwrite pol |] )
     ,$(adaptive[| \ss (v1:vs)-> not ((keyid v1) `elem` (keyids ss)) |])
         ==> \ss (v:_) -> v : ss
     ,$(adaptive[| \ss (v1:vs)-> (keyid v1) `elem` (keyids ss) |])
         ==> \ss (v:_) -> uncurry (:) (extract (keyid v) ss)
   ]
{-
tmp::[Object]
tmp =[ [("PatientID",Str "64182B95-EB72-4E2B-BE77-8050B71498CE"),("AdmissionID",Str "1"),("labName",Str "METABOLIC"),("LabValue",Str "8.9"),("LabUnits",Str "m/cumm"),("LabDateTime",Str "")]]

tmp1::[Object]
tmp1 = [[("PatientID",Str "64182B95-EB72-4E2B-BE77-8050B71498CE"),("AdmissionID",Str "1"),("labName",Str "METABOLIC"),("LabValue",Str "8.9"),("LabUnits",Str "m/cumm"),("LabDateTime",Str "sds")]]

tmpS::[Object]
tmpS = [[("PatientID",Str "FB2ABB23-C9D0-4D09-8464-49BF0B982F0F"),("AdmissionID",Str "7"),("labName",Str "CBC"),("LabValue",Str "1.8"),("LabUnits",Str "pg"),("LabDateTime",Str "rbc/hpf1992-07-0101")],[("PatientID",Str "64182B95-EB72-4E2B-BE77-8050B71498CE"),("AdmissionID",Str "1"),("labName",Str "METABOLIC"),("LabValue",Str "8.9"),("LabUnits",Str "m/cumm"),("LabDateTime",Str "")]]

tmpV::[Object]
tmpV = [[("PatientID",Str "FB2ABB23-C9D0-4D09-8464-49BF0B982F0F"),("AdmissionID",Str "7"),("labName",Str "CBC"),("LabValue",Str "1.8"),("LabUnits",Str "pg"),("LabDateTime",Str "hpf1992-07-0101")]]
-}

