{-# LANGUAGE FlexibleContexts, TemplateHaskell, TypeFamilies, ScopedTypeVariables #-}
module JsonFormat where

import Generics.BiGUL
import Generics.BiGUL.Interpreter
import Generics.BiGUL.TH
import Generics.BiGUL.Lib
import Data.List
import Data.Maybe
import GHC.Generics
import Debug.Trace
import Structure
import System.IO  
import Control.Monad
import Text.Regex


test_JsonFormat = do  
        handle <- openFile "100-Patients-2k-labs.json" ReadMode
        contents <- hGetContents handle
        let wordslist = words contents
            str = concat wordslist
        print str
        let bis = paserobject str
        print bis
        writeFile "result.txt" "this is the json:\n"
        appendFile "result.txt" (showobject bis 0)
        hClose handle


findmatchedbracket::String -> Int -> Int
findmatchedbracket str i | length str == 0 = error "given string is not legal when matching pair bracket"
                         | i==1 && (take 1 str) == "}" = 1
                         | i/=1 && (take 1 str) == "}" = 1 + findmatchedbracket (drop 1 str) (i-1)
                         | (take 1 str == "{") = 1 + findmatchedbracket (drop 1 str) (i+1)
                         | otherwise = 1 + findmatchedbracket (drop 1 str) i

findmatchedarray::String -> Int -> Int
findmatchedarray str i | length str == 0 = error "given string is not legal in array bracket"
                       | i==1 && (take 1 str) == "]" = 1
                       | i/=1 && (take 1 str) == "]" = 1 + findmatchedarray (drop 1 str) (i-1)
                       | (take 1 str == "[") = 1 + findmatchedarray (drop 1 str) (i+1)
                       | otherwise = 1 + findmatchedarray (drop 1 str) i

findmatchedquote::String -> Int -> Int
findmatchedquote str i | length str == 0 = error "given string is not legal when matching \""
                       | i==0 && (take 1 str) == "\"" = 1 + findmatchedquote (drop 1 str) 1
                       | i==1 && (take 1 str) == "\"" = 1
                       | otherwise = 1 + findmatchedquote (drop 1 str) i

findfirstseparator :: String -> Int
findfirstseparator str | length str == 0 = 1
                       | (take 1 str) == "," = 1
                       | (take 1 str) == "{" = x + findfirstseparator (drop x str) 
                       | (take 1 str) == "[" = y + findfirstseparator (drop y str)
                       | (take 1 str) == "\"" = z + findfirstseparator (drop z str)  
                       | otherwise = 1 + findfirstseparator (drop 1 str)
                           where x = findmatchedbracket str 0
                                 y = findmatchedarray str 0 
                                 z = findmatchedquote str 0


paserobject :: String -> Object
paserobject str = keyvaluepairs
                  where keyvaluepairs = paserkeyvalues (drop 1 (take (x-1) str))
                        x = findmatchedbracket str 0


paserkeyvalues :: String -> [(Key, Value)]
paserkeyvalues str | length str == 0 = []
                   | otherwise = paserkeyvalue (take (x-1) str) : paserkeyvalues (drop x str)
                     where x = findfirstseparator str



paserkeyvalue :: String -> (Key, Value)
paserkeyvalue str = (key, value)
                    where key = takeWhile (/='"') (drop 1 (takeWhile (/=':') str))
                          value = paservalue (drop 1 (dropWhile (/=':') str))

paservalue :: String -> Value
paservalue str | (take 1 str) == "\"" = Str (takeWhile (/='"') (drop 1 (takeWhile (/=':') str)))
               | (take 2 str) == "[\"" = Arrs (paserarraystring (drop 1 (take (x-1) str)))
               | (take 2 str) == "[{" = Arro (paserarrayobject (drop 1 (take (x-1) str)))
               | otherwise = Obj (paserobject str)
                 where x = findmatchedarray str 0

paserarraystring :: String -> [String]
paserarraystring str = splitRegex (mkRegex ",") str

paserarrayobject :: String -> [Object]
paserarrayobject str | length str == 0 = []
                     | otherwise = (paserobject (take (x-1) str)) : (paserarrayobject (drop x str))
                       where x = findfirstseparator str 

showstrings :: [String]->String
showstrings strs | length strs == 0 = ""
                 | length strs == 1 = head strs
                 | otherwise = head strs ++ showstringssecond (tail strs)

showstringssecond :: [String]->String
showstringssecond strs | length strs == 0 = ""
                       | otherwise = ","++ head strs ++ showstringssecond (tail strs)

showspace :: Int -> String
showspace i | i== 0 = ""
            | otherwise = "   " ++ (showspace (i-1))

showobject :: Object -> Int-> String
showobject obj layer = (showspace layer) ++ "{\n" ++ (showkeyvaluepairs obj (layer+1)) ++ (showspace layer) ++"}\n"


showarrayobject :: [Object] -> Int -> String
showarrayobject objs layer | length objs == 0 = ""
                           | length objs == 1 = (showobject (head objs) layer)
                           | otherwise = (showobject (head objs) layer) ++ (showarrayobjectsecond (tail objs) layer)

showarrayobjectsecond :: [Object] -> Int -> String
showarrayobjectsecond objs layer | length objs == 0 = ""
                                 | otherwise = ","++(showobject (head objs) layer) ++ (showarrayobjectsecond (tail objs) layer)

showkeyvaluepairs :: [(Key, Value)] -> Int -> String
showkeyvaluepairs pairs layer 
                | length pairs == 0 = ""
                | length pairs == 1 = (showkeyvaluepair (head pairs) layer )
                | otherwise = (showkeyvaluepair (head pairs) layer ) ++ (showkeyvaluepairssecond (tail pairs) layer)

showkeyvaluepairssecond :: [(Key, Value)] -> Int -> String
showkeyvaluepairssecond pairs layer 
                | length pairs == 0 = ""
                | otherwise = ","++(showkeyvaluepair (head pairs) layer ) ++ (showkeyvaluepairssecond (tail pairs) layer)

showkeyvaluepair :: (Key, Value) -> Int -> String
showkeyvaluepair keyvalue layer = case (snd keyvalue) of
                       Obj o -> (showspace layer)++"\""++(fst keyvalue)++"\":\n" ++ (showobject o layer)
                       Str s -> (showspace layer)++"\""++(fst keyvalue)++"\":\""++s++"\"\n" 
                       Arrs as -> (showspace layer)++"\""++(fst keyvalue)++"\":["++showstrings as++"]\n" 
                       Arro os -> (showspace layer)++"\""++(fst keyvalue)++"\":\n"++(showspace layer)++"[\n"++(showarrayobject os (layer+1))++(showspace layer)++"]\n"

